/* 
 * This file is part of the Echo Point Project.  This project is a collection
 * of Components that have extended the Echo Web Application Framework.
 *
 * EchoPoint is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * EchoPoint is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Echo Point; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//========================================================================================================
// Object maps
//========================================================================================================
var EchoSelectableTableMap = new Array();

//========================================================================================================
// The EchoSelectableTable object
//========================================================================================================
function EchoSelectableTable(htmlId, rowCount, columnCount, multiSelect, stateStr, useClassNames, selectedClass, defaultClass, rolloverClass, selectedBackground, defaultBackground, rolloverBackground) {
	// members
	this.htmlId 			= htmlId;
	this.rowCount 			= rowCount;
	this.columnCount 		= columnCount;
	this.multiSelect 		= multiSelect;
	this.useClassNames		= useClassNames;
	this.selectedClass 		= selectedClass;
	this.defaultClass 		= defaultClass;
	this.rolloverClass 		= rolloverClass;
	this.selectedBackground = selectedBackground; 
	this.defaultBackground 	= defaultBackground; 
	this.rolloverBackground = rolloverBackground;
	
	this.cellBackgrounds 	= new Array(rowCount); 

	this.selectedRows 		= stateStr.split('|');
	// and convert to boolean
	for (var j = 0; j < this.selectedRows.length; j++) {
		if (this.selectedRows[j] == 'true')
			this.selectedRows[j] = true;
		else
			this.selectedRows[j] = false;
	}
	EchoSelectableTableMap[htmlId] = this;
}


//---------------------------------------------------------------------------------------
// This is called to SAVE the backgrouns of selected rows at load time.  That
// way we have a an original "copy" so they can be later restored.  Without
// this there is no way to know what it should be after a selection or rollover
// has taken place.
//---------------------------------------------------------------------------------------
EchoSelectableTable.prototype.init = function() {
	var trE = null;
	for (row = 0; row < this.rowCount; row++) {
		//
		// if we are selected then set the selection color
		if (this.selectedRows[row]) {
			this.setRowBackground(row,this.selectedBackground,this.selectedClass);
		}
	}
}

//---------------------------------------------------------------------------------------
// Called to set the background color of row.  Saves the original
// background if is not already done.
//---------------------------------------------------------------------------------------
EchoSelectableTable.prototype.setRowBackground = function(row, newBG, className) {
	var trId = this.htmlId + "_" + row;
	var trE = getElementById(trId);
	if (trE == null) 
		alert("ASSERT : SelectTable : TR not found : " + trId);
	//
	// PERFORMANCE ISSUE - Setting both the background and classname
	// slows things down very noticeably in IE.  OK in Moz
	// So we do one or the other but not both.
	// 
	if (this.useClassNames) {
		trE.htmlE.className = className;
	} else {
		//
		// if we havent encountered this row before then save its background.
		// This "late save strategy"  saves an up front load time of saving
		// of all backgrounds first.  For large row counts its way too slow.
		// But if the user nevers "rolls over" or "selects" a row then
		// we dont take a hit.
		//
		var doSave = false;
		if (this.cellBackgrounds[row] == null)
			doSave = true;
		
		var saveArea = epst_setElementBackgrounds(trE.htmlE,newBG,doSave);
		if (doSave)
			this.cellBackgrounds[row] = saveArea;
	}
}

//---------------------------------------------------------------------------------------
// Called to restore the background color of row from the original saved version
//---------------------------------------------------------------------------------------
EchoSelectableTable.prototype.restoreRowBackground = function(row, className) {
	var trId = this.htmlId + "_" + row;
	var trE = getElementById(trId);
	if (trE == null) 
		alert("ASSERT : SelectTable : TR not found : " + trId);
	//
	// PERFORMANCE ISSUE - Setting both the background and classname
	// slows things down very noticeably in IE.  OK in Moz.
	// So we do one or the other but not both.
	// 
	if (this.useClassNames) {
		trE.htmlE.className = className;
	} else {
		var saveArea = this.cellBackgrounds[row];
		epst_restoreElementBackgrounds(trE.htmlE,saveArea);
	}
}


//---------------------------------------------------------------------------------------
// a string representing the state of our selections
//---------------------------------------------------------------------------------------
EchoSelectableTable.prototype.makeStateStr = function() {
	var str = '';
	for (i = 0; i < this.selectedRows.length; i++) {
		str += this.selectedRows[i];
		if (i < this.selectedRows.length - 1)
			str += '|';
	}
	return str;
}

//---------------------------------------------------------------------------------------
// saves the state of the selectable table.
//---------------------------------------------------------------------------------------
EchoSelectableTable.prototype.saveState = function() {
	var sbpos = '';
	var e = getElementById(this.htmlId);
	if (e != null)
		sbpos = e.htmlE.scrollLeft + "," + e.htmlE.scrollTop;
	else
		sbpos = "0,0";

	var stateStr = sbpos + '&' + this.makeStateStr();

	if (E_setParameter)
		E_setParameter(this.htmlId, stateStr);
}

//-------------------------------------------------
//-------------------------------------------------
EchoSelectableTable.prototype.toggle = function(row, columnCount) {
	var i = 0;
	//
	// set the state of the item
	var isSelected = this.selectedRows[row];
	isSelected = !isSelected;
	this.selectedRows[row] = isSelected;
	//
	// we might only allow one selection
	if (!this.multiSelect) {
		var oldSelected = -1;
		for (i = 0; i < this.selectedRows.length; i++) {
			if (i != row) {
				if (this.selectedRows[i]) {
					this.selectedRows[i] = false;
					oldSelected = i;
					break;
				}
			}
		}
		if (oldSelected != row && oldSelected != -1) {
			var elem = getElementById(this.htmlId + "_" + oldSelected);
			var chk = getElementById(this.htmlId + "_chk_" + oldSelected);
			if (elem != null) {
				this.restoreRowBackground(oldSelected,this.defaultClass);
			}
			if (chk != null)
				chk.htmlE.checked = false;
		}
	}
	//
	// now update to new visual repesentation
	var trE = getElementById(this.htmlId + "_" + row);
	var chk = getElementById(this.htmlId + "_chk_" + row);
	if (trE != null) {
		// if we are changing state then the mouse must be over us
		// and hence we should use the rollover if it is valid
		if (this.rolloverClass != null && this.rolloverBackground != null) {
			this.setRowBackground(row,this.rolloverBackground, this.rolloverClass);
		} else {
			if (isSelected) {
				this.setRowBackground(row,this.selectedBackground, this.selectedClass);
			} else {
				this.restoreRowBackground(row, this.defaultClass);
			}
		}
	}
	if (chk != null)
		chk.htmlE.checked = isSelected;
	this.saveState();
}

//-------------------------------------------------
// Called to select a row in a table
//-------------------------------------------------
function g_epst_click(event, element, htmlId, row, columnCount) {
	var stbl = EchoSelectableTableMap[htmlId];
	if (stbl != null) {
		stbl.toggle(row, columnCount);
	}
}

//-------------------------------------------------
//-------------------------------------------------
function g_epst_saveScrollBarPositions(htmlId) {
	var stbl = EchoSelectableTableMap[htmlId];
	if (stbl != null)
		stbl.saveState();
}

//-------------------------------------------------
// called when the mouse moves out of a rollover
// table row.  
//-------------------------------------------------
function g_epst_mouseout(element, htmlId, row) {
	var stbl = EchoSelectableTableMap[htmlId];
	if (stbl != null) {
		if (stbl.selectedRows[row]) {
			stbl.setRowBackground(row,stbl.selectedBackground, stbl.selectedClass);
		} else {
			stbl.restoreRowBackground(row, stbl.defaultClass);
		}
	}
}

//-------------------------------------------------
// Called when the mouse goes over a table row
//-------------------------------------------------
function g_epst_mouseover(element, htmlId, row) {
	var stbl = EchoSelectableTableMap[htmlId];
	if (stbl != null) {
		if (stbl.rolloverClass) {
			stbl.setRowBackground(row,stbl.rolloverBackground);
		}
	}		
}

//-------------------------------------------------
// Called when the browser is IE and it supports
// expression CSS properties.
//-------------------------------------------------
function g_epst_synchTop(htmlId, borderInPlay) {
	var e = getElementById(htmlId);
	var top = e.htmlE.scrollTop;
	if (borderInPlay)
		top = top - 1;
	return top;
}

//---------------------------------------------------------------------------------------
// Called on document load
//---------------------------------------------------------------------------------------
function g_epst_onLoad(htmlId) {
	var stbl = EchoSelectableTableMap[htmlId];
	if (stbl != null)
		stbl.init();
}

//--------------------------------------------
// Object class needed to save an element and 
// its background color.
//--------------------------------------------
function EPElementBackground(e,bg) {
	this.e = e;
	this.bg = bg;
}

//--------------------------------------------
// Sets all elements to a specified background
// and returns an array of the old values
// which is an array of EPElementBackground
// objects, one for each element in the 
// hierarchy.  If the new background is null
// then no setting takes place but the old
// backgrounds are still saved.
//--------------------------------------------
function epst_setElementBackgrounds(parentElem,newBG,doSave) {
	var elementBackgroundArray = null;
	if (doSave)
		elementBackgroundArray = new Array();
	epst_setElementBG(parentElem,elementBackgroundArray,newBG);
	return elementBackgroundArray;
}

//--------------------------------------------
// Recursive function that saves and then sets
// the element background color if told to!
//--------------------------------------------
function epst_setElementBG(e,elementBackgroundArray,newBG) {
	var bg;
	var child;
	var style;
	style = epst_getElementStyleWithBackground(e);
	if (style && style.backgroundColor) {
		if (elementBackgroundArray) {
			bg = style.backgroundColor;
			var eb = new EPElementBackground(e,bg);
			elementBackgroundArray[elementBackgroundArray.length] = eb;
		}
		if (newBG)
			e.style.backgroundColor = newBG;
	}
	child = e.firstChild;
	while (child) {
		if (child && child.nodeType == 1) {
			epst_setElementBG(child,elementBackgroundArray,newBG)
		}
		child = child.nextSibling;
	}
}

//--------------------------------------------
// This function will restore the background
// colors of the elements in the elementBackgroundArray
// which contains EPElementBackground objects
//--------------------------------------------
function epst_restoreElementBackgrounds(parentElem, elementBackgroundArray) {
	epst_restoreElementBG(parentElem,elementBackgroundArray);
}

//--------------------------------------------
// Recursive function that restores the element 
// background colors.
//--------------------------------------------
function epst_restoreElementBG(e,elementBackgroundArray) {
	var child;
	var bg;

	for (i=0; i < elementBackgroundArray.length;i++) {
		if (elementBackgroundArray[i].e == e) {
			bg = elementBackgroundArray[i].bg;
			e.style.backgroundColor = bg;
			break;
		}
	}
	child = e.firstChild;
	while (child) {
		if (child && child.nodeType == 1) {
			epst_restoreElementBG(child,elementBackgroundArray)
		}
		child = child.nextSibling;
	}
}

//--------------------------------------------
// Returns the style object for the element that contains
// the background value if any.  Done in proper style 
// application order
//--------------------------------------------
function epst_getElementStyleWithBackground(elem) {
	if (elem.style && elem.style.backgroundColor)
		return elem.style;

	if (elem.className) {
		var style = epst_getStyleFromStyleSheets(elem.className);
		if (style && style.backgroundColor)
			return style;
	}
	return null;
}

//--------------------------------------------
// Returns the style object with the specified name 
// from document style sheets or null if it can find 
// it.
//--------------------------------------------
function epst_getStyleFromStyleSheets(styleName) {
	var style = null;
	var i = 0;
	var rules;
	var length;
	var testName = styleName.toLowerCase();
	var name;
	var dotIndex;
    
	if (document.styleSheets && document.styleSheets[0]) {
		if (document.styleSheets[0].cssRules) {
			rules = document.styleSheets[0].cssRules;
		} else if (document.styleSheets[0].rules) {
			rules = document.styleSheets[0].rules;
		}
	}
	if (rules) {
		length = rules.length;
		while (i < length && style == null) {
			name = rules[i].selectorText.toLowerCase();
			dotIndex = name.indexOf(".");
			if (dotIndex >= 0) {
				name = name.substring(dotIndex + 1);
			}
			if (name == testName) {
				style = rules[i].style;
			}
			++i;
		}
	}
	return style;
}