/* 
 * This file is part of the Echo Point Project.  This project is a collection
 * of Components that have extended the Echo Web Application Framework.
 *
 * EchoPoint is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * EchoPoint is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Echo Point; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//========================================================================================================
// Object maps
//========================================================================================================
var EchoTitleBarMap = new Array();

//========================================================================================================
// The EchoToggleImage object
//
// Used to hold an image reference that needs to be toggled when the title bar is toggled
//========================================================================================================
function EchoToggleImage(htmlId, expandImgURI, collapseImgURI) {
	this.htmlId				= htmlId;

	this.expandImg			= new Image();
	this.expandImg.src		= expandImgURI;

	this.collapseImg		= new Image();
	this.collapseImg.src	= collapseImgURI;
	
	// functions
	this.toggle				= ti_toggle;
}

//---------------------------------------------
// Called to toggle the images
//---------------------------------------------
function ti_toggle(expansionState) {
	var imgElement = getElementById(this.htmlId);
	if (imgElement == null)
		return;
	if (expansionState) {
		imgElement.htmlE.src = this.expandImg.src;
	} else {
		imgElement.htmlE.src = this.collapseImg.src;
	}	
}

//========================================================================================================
//
// The EchoTitleBar object
//
//========================================================================================================
function EchoTitleBar(htmlId, expansionState, hideHtmlId) {
	// members
	this.htmlId				= htmlId;
	this.expansionState		= expansionState;
	this.hideHtmlId			= null;
	if (hideHtmlId != null)
		this.hideHtmlId		= hideHtmlId;

	this.toggleImages 			= new Array();
	// functions
	this.toggle 				= eptb_toggle;
	this.addImage 				= eptb_addImage;

	EchoTitleBarMap[htmlId] = this;
}


//---------------------------------------------
// Called to add a EchoToggleImage to the TitleBar
//---------------------------------------------
function eptb_addImage(eti) {
	this.toggleImages[this.toggleImages.length] = eti;
}

//---------------------------------------------
// Called to toggle the state of the TitleBar
//---------------------------------------------
function eptb_toggle() {
	this.expansionState = ! this.expansionState;
	
	for (i = 0; i < this.toggleImages.length; i++ ) {
		this.toggleImages[i].toggle(this.expansionState);
	}
	
	var stateStr = 'false';
	if (this.expansionState)
		stateStr = 'true';
	
	//
	// we can also hide a html element if necessary
	if (this.hideHtmlId != null) {
		var elem = getElementById(this.hideHtmlId);
		if (elem != null) {
			if (this.expansionState) {
				elem.htmlE.style.display = '';
			} else {
				elem.htmlE.style.display = 'none';
			}
		}
	}
	//
	// save our client side state
	if (E_setParameter)
		E_setParameter(this.htmlId,stateStr);
}

//---------------------------------------------
// Global function to toggle the titlebar
//---------------------------------------------
function g_titlebar_toggle(htmlId) {
	var tb = EchoTitleBarMap[htmlId];
	if (tb == null)
		return;
	tb.toggle();
}

