/*
 * This file is part of the SOAFaces Project (hereinafter "SOAFaces").
 * Copyright (C) 2006 Grand Logic, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */


package org.soafaces.bundle.client.rpc.impl;

import com.google.gwt.core.client.GWT;
import com.google.gwt.i18n.client.Dictionary;
import com.google.gwt.json.client.JSONObject;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.IsSerializable;
import com.google.gwt.user.client.ui.RootPanel;
import java.util.Map;
import org.soafaces.bundle.client.rpc.AppCookie;
import org.soafaces.bundle.client.rpc.FetchAppCookieCallback;
import org.soafaces.bundle.client.rpc.FetchTaskletStatusCallback;
import org.soafaces.services.client.rpc.UniversalClient;
import org.soafaces.bundle.client.rpc.FetchJSONCallback;
import org.soafaces.bundle.client.rpc.FetchPOJOCallback;
import org.soafaces.bundle.client.rpc.IOViewerContext;
import org.soafaces.bundle.client.rpc.WebletContext;
import org.soafaces.bundle.client.rpc.SuccessFailCallback;
import org.soafaces.bundle.client.ui.Weblet;

/**
 *
 * @author Sam Taha
 */
class BundleDataAccess implements WebletContext, IOViewerContext {

  private static final int NOT_FETCH_YET=0;
  private static final int POJO_BEAN_FETCH=1;
  private static final int JSON_BEAN_FETCH=2;

  private BundleDataAccessRPCAsync _oTaskbeanService = null;
  private String _stContextName = null;
  private Weblet.Mode _oWebletMode = Weblet.Mode.DEMO; // 1:Design 2:Application edit 3:App view  4:DEMO
  private String _stUsername=null;
  private long _lJobId=-1;
  private long _lRunId=-1;
  private long _lTaskId=-1;
  private long _lEffDate=-1;
  private boolean _bIsApp=true;
  private boolean _bIsRetryTasklet=true;
  private String _stTaskType = null; //eg: com-beantest-MyBaen-v1-0-0
  private String _stUIType = null;
  private BundleUniversalClient _oUniversalClientAccess = null;

  private AppCookie _oAppCookie = null;
  private IsSerializable _oSaveInputBeanPOJO = null;
  private JSONObject _oSaveInputBeanJSON = null;
  private int _iSaveInputBeanFetchType=NOT_FETCH_YET;

  private Weblet _oCustomizer = null;
  
  private BundleDataAccess() { }

  public BundleDataAccess (String contextName, 
                             int mode, long jobid, long runid, long taskid, boolean bIsApp,/* boolean bIsRetryTasklet,*/
                             long lEffDate, String stUIType, String stUsername,
                             String stTaskType, BundleUniversalClient muleAccess) 
  {
    _bIsApp=bIsApp;
    //_bIsRetryTasklet=bIsRetryTasklet;
    _stContextName = contextName; //should be just "jobserver"
    _stUsername=stUsername;
    
    //if mode is for ioviewer then it really does not matter since this var will not be used
    _oWebletMode = MapWebletViewerEnums.getWebletEnum(mode);
    _lJobId = jobid;
    _lRunId = runid;
    _lTaskId = taskid;
    _lEffDate = lEffDate;
    _oTaskbeanService = (BundleDataAccessRPCAsync) GWT.create(BundleDataAccessRPC.class);
    _stTaskType = stTaskType;
    _stUIType = stUIType;
    _oUniversalClientAccess = muleAccess;
     
    // (2) Specify the URL at which our service implementation is running.
    // Note that the target URL must reside on the same domain and port from
    // which the host page was served.
    //
    ServiceDefTarget endpoint = (ServiceDefTarget) _oTaskbeanService;
    String moduleRelativeURL = "/" + _stContextName  + "/rpcservices/taskbeanservice?tasktype=" +_stTaskType;
    endpoint.setServiceEntryPoint(moduleRelativeURL);
    
    // (3) Create an asynchronous callback to handle the result.
  }

  void attachedAppCookie(AppCookie appCookieMap)
  {
    _oAppCookie = appCookieMap;
  }

  void attachedInputBeanPOJO(IsSerializable inputBean) { _oSaveInputBeanPOJO = inputBean; }
  
  void attachedInputBeanJSON(JSONObject inputBean) { _oSaveInputBeanJSON = inputBean; }
  
  void associateCustomizer(Weblet customizer) { _oCustomizer = customizer; }
  
  /**
   * Save input javabean back to the server. Called by javascript
   * 
   * @return 0 if ok, -1 if there is a server side problem, -2 validation error
   */
  public int saveInputBeanInternal() 
  {
    if(_oWebletMode == Weblet.Mode.READONLY ||
       _oWebletMode == Weblet.Mode.DEMO)
      return 0; //only save if design or app edit mode
    
    if(_oSaveInputBeanPOJO == null && _oSaveInputBeanJSON == null)
    {
      //really nothing to save so just return control back to the javascript thing that called me
      ApplicationBoot.echoSaveAndReturn();
      return 0;
    }
  
    if(_iSaveInputBeanFetchType==NOT_FETCH_YET)
      return 0;
    
    SuccessFailCallback relayCallback = new SuccessFailCallback() {
      protected void onSuccess()
      {
        realSaveInputBeanInternal();
      }
      
      protected void onFailure(String errorMsg)
      {      
        Window.alert("Error: Problem validating/saving input JavaBean : " + errorMsg);
      }
      
      protected void onFailure() {
        //Window.alert("Error: Problem validating/saving input JavaBean");
      }
    };
    
    try {
      _oCustomizer.onSaveInputBean(relayCallback);
    }catch(Throwable ex) {
      //this try/catch is only good for foreground processing. It is up to the
      //Customizer to throw success/failure if something goes wrong with async calls.
      relayCallback.returnFailure(ex.getMessage());
    }
    
    return 0;
  }
   
  private void realSaveInputBeanInternal()
  {
    
    AsyncCallback callback = new AsyncCallback() {
      public void onSuccess(Object result) {

        ApplicationBoot.echoSaveAndReturn();
        if(RootPanel.get("go-loadingdiv") != null)
          RootPanel.get("go-loadingdiv").setVisible(false);
      }

      public void onFailure(Throwable caught) {
        // do some UI stuff to show failure
        //should probably do a javascript window since this is happening async
        //the screen will move back assuming it was a success
        if(RootPanel.get("go-loadingdiv") != null)
          RootPanel.get("go-loadingdiv").setVisible(false);
        //PopupPanel popup = new MsgPopupPanel("RPC saveInputBean error: ", caught);
        Window.alert("Error: " + "RPC saveInputBean error - " + caught.getMessage());
      }
    };
    
    //Window.alert("_lTaskId 2 =" + _lTaskId);
    
    if(RootPanel.get("go-loadingdiv") != null)
      RootPanel.get("go-loadingdiv").setVisible(true);
    if(_iSaveInputBeanFetchType==JSON_BEAN_FETCH)
    {
      String jsonString = _oSaveInputBeanJSON.toString();
      _oTaskbeanService.setInputBeanJSON(_lJobId, _lTaskId, _bIsApp, jsonString, _stTaskType, _lEffDate, callback);
    }
    else
    {
      //TypeProxy proxy = TypeProxyUtil.createTypeProxy(_oSaveInputBeanPOJO);
      _oTaskbeanService.setInputBeanPOJO(_lJobId, _lTaskId, _bIsApp, _oSaveInputBeanPOJO, _stTaskType, _lEffDate, callback);
    }
  }
  
  /**
   * Save input javabean back to the server. Called by javascript
   * 
   * @return 0 if ok, -1 if there is a server side problem, -2 validation error
   */
  public int saveCacheInputBean() 
  {
    if(_oSaveInputBeanPOJO == null && _oSaveInputBeanJSON == null)
      return 0;
    
    if(_oWebletMode == Weblet.Mode.READONLY ||
       _oWebletMode == Weblet.Mode.DEMO)
      return 0; //only save if design or app edit mode
  
    if(_iSaveInputBeanFetchType==NOT_FETCH_YET)
      return 0;
    
    SuccessFailCallback relayCallback = new SuccessFailCallback() {
      protected void onSuccess()
      {
        realSaveCacheInputBean();
      }
      
      protected void onFailure(String errorMsg)
      {
        Window.alert("Error: Problem validating/saving input JavaBean : " + errorMsg);
      }
      
      protected void onFailure() {
        //Window.alert("Error: Problem validating/saving input JavaBean");
      }
    };
 
    try {
      _oCustomizer.onSaveInputBean(relayCallback);
    }catch(Throwable ex) {
      //this try/catch is only good for foreground processing. It is up to the
      //Customizer to throw success/failure if something goes wrong with async calls.
      relayCallback.returnFailure(ex.getMessage());
    }
    
    return 0;
  }
  
  private void realSaveCacheInputBean()
  {
    
    AsyncCallback callback = new AsyncCallback() {
      public void onSuccess(Object result) {

        ApplicationBoot.echoSaveAndReturn();
        if(RootPanel.get("go-loadingdiv") != null)
          RootPanel.get("go-loadingdiv").setVisible(false);
      }

      public void onFailure(Throwable caught) {
        // do some UI stuff to show failure
        //should probably do a javascript window since this is happening async
        //the screen will move back assuming it was a success
        if(RootPanel.get("go-loadingdiv") != null)
          RootPanel.get("go-loadingdiv").setVisible(false);
        //PopupPanel popup = new MsgPopupPanel("RPC saveInputBean error: ", caught);
        Window.alert("Error: " + "RPC saveInputBean error - " + caught.getMessage());
      }
    };
    
    if(RootPanel.get("go-loadingdiv") != null)
          RootPanel.get("go-loadingdiv").setVisible(true);
    if(_iSaveInputBeanFetchType==JSON_BEAN_FETCH)
    {
      String jsonString = _oSaveInputBeanJSON.toString();
      _oTaskbeanService.cacheTmpInputBeanJSON(_lJobId, _lTaskId, jsonString, callback);
    }
    else
    {
      //GenericDataWrapper proxy = GenericDataUtil.createGenericDataWrapper(_oSaveInputBeanPOJO);
      _oTaskbeanService.cacheTmpInputBeanPOJO(_lJobId, _lTaskId, _oSaveInputBeanPOJO, callback);
    }
  }

  public UniversalClient getUniversalClient()
  { 
    return _oUniversalClientAccess;
  }
  public String getUsername() { return _stUsername; }
  public long getID() 
  { 
    return _lJobId;
  }
  public String getName() { return null; }
  
  public long getBundleID() 
  {
    return _lTaskId; 
  }
  public String getBundleName() { return null; }
  
  public long getTaskletID() {
    return _lTaskId;
  }

  public String getTaskletName() {
    return null;
  }
  
  public long getJobRunID() { return _lRunId; } 
  
  public long getJobID() {
    return _lJobId;
  }

  public String getJobName() {
    return null;
  }
  
  /**
   * Will save JSON vs IsSerializable depending on what was initally fetched
   * @param customizer
   * @param callback
   * @return 0 request submitted to server, 1 validation error, 2 input bean has not been fetched 
   *         thus nothing to save, Read only mode - cannot save inputbean
   */
  public void saveInputBean(AsyncCallback callback)
  {
    if(_oSaveInputBeanPOJO == null && _oSaveInputBeanJSON == null)
      callback.onSuccess(null);
    
    if(_oWebletMode == Weblet.Mode.READONLY ||
       _oWebletMode == Weblet.Mode.DEMO)
      callback.onSuccess(null);
  
    if(_iSaveInputBeanFetchType==NOT_FETCH_YET)
      callback.onSuccess(null);
    
    final AsyncCallback myCallback = callback;
    SuccessFailCallback relayCallback = new SuccessFailCallback() {
      protected void onSuccess()
      {
        realSaveInputBean(myCallback);
      }
      
      protected void onFailure(String errorMsg)
      {
        //myCallback.onFailure(new ValidateInputBeanException(errorMsg));
        Window.alert("Error: Problem validating/saving input JavaBean : " + errorMsg);
      }
      
      protected void onFailure() {
        //myCallback.onFailure(new ValidateInputBeanException());
        //Window.alert("Error: Problem validating/saving input JavaBean : " + errorMsg);
      }
    };
 
    _oCustomizer.onSaveInputBean(relayCallback);
  }
  
  //public void setValidateInputBeanExceptionHanlder(AsyncCallback callback)
  //{
  //  _oValidateInputBeanExceptionHandler = callback;
  //}
    
  private void realSaveInputBean(AsyncCallback callback)
  {
    if(_iSaveInputBeanFetchType==JSON_BEAN_FETCH)
    {
      String jsonString = _oSaveInputBeanJSON.toString();
      _oTaskbeanService.setInputBeanJSON(_lJobId, _lTaskId, _bIsApp, jsonString, _stTaskType, _lEffDate, callback);
    }
    else
    {
      _oTaskbeanService.setInputBeanPOJO(_lJobId, _lTaskId, _bIsApp, _oSaveInputBeanPOJO, _stTaskType, _lEffDate, callback);
    }
  }
  public void fetchInputBean(FetchPOJOCallback callback) 
  {
    if(_stUIType.equals(ApplicationBoot.WEBLET))
    {     
      AsyncCallback proxyCallback = new ProxyPOJOAsyncCallback(callback, this);  
      
      _iSaveInputBeanFetchType=POJO_BEAN_FETCH;
      _oTaskbeanService.getInputBeanPOJO(_lJobId, _lTaskId, _stTaskType, _lEffDate, proxyCallback);
    }
    else //input runtime bean
    {
      _oTaskbeanService.getInputBeanRuntimePOJO(_lJobId, _lRunId, _lTaskId, _stTaskType, callback);
    }
  }
  public void fetchInputBean(FetchJSONCallback callback) 
  {
    if(_stUIType.equals(ApplicationBoot.WEBLET))
    {
      
      AsyncCallback proxyCallback = new ProxyJSONAsyncCallback(callback, this);
      _iSaveInputBeanFetchType=JSON_BEAN_FETCH;
      _oTaskbeanService.getInputBeanJSON(_lJobId, _lTaskId, _stTaskType, _lEffDate, proxyCallback);
    }
    else //input runtime bean
    {
      _oTaskbeanService.getInputBeanRuntimeJSON(_lJobId, _lRunId, _lTaskId, _stTaskType, callback);
    }
  }
  public void fetchOutputBean(FetchPOJOCallback callback) 
  {
    _oTaskbeanService.getOutputBeanRuntimePOJO(_lJobId, _lRunId, _lTaskId, _stTaskType, callback);
  }
  public void fetchOutputBean(FetchJSONCallback callback) 
  {
    _oTaskbeanService.getOutputBeanRuntimeJSON(_lJobId, _lRunId, _lTaskId, _stTaskType, callback);
  }
  
  public Dictionary getURLParameters()
  {
    return Dictionary.getDictionary("SOAFacesURLParams");
  }

  public boolean isRetry()
  {
    return _bIsRetryTasklet;
  }

  public void fetchTaskletStatus(FetchTaskletStatusCallback callback)
  {
    _oTaskbeanService.getTaskletStatus(_lJobId, _lRunId, _lTaskId, callback);
  }

  public void fetchAppCookie(FetchAppCookieCallback callback)
  {
    AsyncCallback proxyCallback = new ProxyAppCookieCallback(callback, this);
    _oTaskbeanService.getAppCookie(_lJobId, _lTaskId, proxyCallback);
  }

  public void saveAppCookie(FetchAppCookieCallback callback)
  {
    AsyncCallback proxyCallback = new ProxyAppCookieCallback(callback, this);

    Map<String,String> editMap = null;
    AppCookieImpl impl = (AppCookieImpl) _oAppCookie;
    if(impl != null)
      editMap = impl.getEditCookieMap();
    _oTaskbeanService.setAppCookie(_lJobId, _lTaskId, editMap, proxyCallback);
  }
}
