/*
 * This file is part of the SOAFaces Project (hereinafter "SOAFaces").
 * Copyright (C) 2006 Grand Logic, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.soafaces.bundle.client.ui;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.user.client.ui.Panel;
import org.soafaces.bundle.client.rpc.IOViewerContext;
import org.soafaces.bundle.client.rpc.impl.ApplicationBoot;

/**
 * <p>
 * Base class used to create a custom viewer for the input/output
 * JavaBeans that are used when a Tasklet is executed. The BaseIOViewer
 * allows the user the ability
 * to view a Tasklet's input or output JavaBean. The BaseIOViewer is used by the user to
 * determine the exact input/output used by the Tasklet when the job/workflow was
 * run.
 * </p>
 * <p>
 * This class functions as the top level GUI container for
 * viewing input and ouput properties and status of a Tasklet as the Tasklet is running
 * or after the Tasklet has finished running. Note, that implementing
 * this class is optional. If a Tasklet has no need for a GUI
 * input/output viewer, to view Tasklets properties, then there
 * is no need to implement this class. 
 * </p>
 * 
 * @author Sam Taha
 * 
 */
@Deprecated
public abstract class BaseIOViewer implements EntryPoint {

	/**
	 * The BaseIOViewer should function fully capable in NORMAL mode. User has access
	 * to all available resources in this mode. DEMO mode is used when run in a demo or 
   * development mode and thus should limit certain functions.
	 */
  public enum Mode {NORMAL, DEMO}

	private IOViewerContext _oContext = null;
	private Mode _iMode = Mode.NORMAL;

  BaseIOViewer() {}
	
	/**
	 * Called by the container to initialize the UI.
	 *
	 * @param context
	 *
   * @param mode Allows the BaseIOViewer to work under <code>NORMAL</code> or <code>DEMO</code> mode.
	 *             DEMO mode typically applies when the BaseIOViewer is used to
   *             demonstrate how the UI works without the actual functionality.
	 */
	final public void initUI(IOViewerContext context, Mode mode) 
	{
    if(_oContext != null)
      return;
		_iMode=mode;
    _oContext=context;
    buildUI();
	}
  
  /**
   * Implement this to create all the UI components that will be housed in this
   * top level container. This is called only once to populte the UI.
   * 
   */
  protected abstract void buildUI();
  
  /**
   * This can return <code>RootPanel.get()</code> or any other panel you want to house your UI.
   * @return
   */
  public abstract Panel getMainPanel();
  
  public final IOViewerContext getContext() { return _oContext; }

	/**	 
	 * Only the more advanced and complex implementations need to concern
	 * themselves with what <code>mode</code> the IOViewer is in. The average
	 * BaseIOViewer will typically not care about this state.
   * 
	 * A subclass should use this property to control how the BaseIOViewer functions.
	 * It is up to the BaseIOViewer to use this property to determine
	 * how to limit its features and functionality.
	 * The possible modes are:
	 *
	 * NORMAL mode<br>
	 * The BaseIOViewer should function fully in this mode. User can access
	 * the input JavaBean and any other resources
	 * such as data stored in the sandbox directory.
	 * 
	 * DEMO mode<br>
	 * Limits what the BaseIOViewer can do.
	 * This mode allows the UI to be rendered to show the user the basic
	 * capabilities of the UI. A default input JavaBean is passed in
	 * and the user should not be able to access any resources.
	 *
	 * @return The mode the MFRuntimeViewer is in.
	 */
	final public Mode getMode()
	{
		return _iMode;
	}

  final public void onModuleLoad() {
    ApplicationBoot app = new ApplicationBoot();
    app.createApplication(this);
  }
}
